local hcTk = HCToolkit
local UnitGUID = UnitGUID

function HCToolkit:OnInitialize()
    -- Called when the addon is loaded
    -- Register Frames
    HCToolkitFrame = CreateFrame("Frame", "HCToolkitFrame", UIParent);
end

function HCToolkit:OnEnable()
    -- Called when the addon is enabled
    -- Register Events
    RegisterAddonMessagePrefix("HCToolkit")
    RegisterAddonMessagePrefix("HCToolkitBoss")
    RegisterAddonMessagePrefix("HCToolkitFlame")
    RegisterAddonMessagePrefix("HCToolkitRoster")

    self:RegisterComm("HCToolkit", "CoreComm")
    self:RegisterComm("HCToolkitBoss", "BossComm")
    self:RegisterComm("HCToolkitFlame", "FlameComm")
    self:RegisterComm("HCToolkitRoster", "RosterComm")

    self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
    self:RegisterEvent("ENCOUNTER_START")
    self:RegisterEvent("ENCOUNTER_END")
    self:RegisterEvent("GROUP_JOINED")
    self:RegisterEvent("GROUP_ROSTER_UPDATE")
    self:RegisterEvent("PLAYER_LOGIN")
    self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
    
    self:RegisterChatCommand("hctk", "ChatCommand")
    self:RegisterChatCommand("HCToolkit", "ChatCommand")

    -- Setup Addon
    hcTk.InitDB()
    hcTk.GeneralInit()
    hcTk.GroupInit()
    hcTk.EncounterInit()
    hcTk.FramesInit()
end

function HCToolkit:OnDisable()
  -- Called when the addon is disabled
end

function HCToolkit:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
    hcTk.GeneralCombatLog(...)
    if (hcTk.EncounterId ~= nil) then
        hcTk.RouteCombatLog(hcTk.EncounterId, ...)
    end
end

function HCToolkit:ENCOUNTER_START(event, encounterId)
    hcTk.DebugPrint("Encounter Start: " .. encounterId)
    hcTk.GeneralEncounterStart(encounterId)
    hcTk.RouteEncounterStart(encounterId)
end

function HCToolkit:ENCOUNTER_END(event, encounterId)
    hcTk.DebugPrint("Encounter End: " .. encounterId)
    hcTk.GeneralEncounterEnd(encounterId)
    hcTk.RouteEncounterEnd(encounterId)
end

function HCToolkit:GROUP_JOINED()
    hcTk.DebugPrint("HC Group Joined")
    hcTk.RequestInformation()
end

function HCToolkit:GROUP_ROSTER_UPDATE()
    hcTk.DebugPrint("HC Group Roster Update")
    hcTk.GroupUpdate()
end

function HCToolkit:PLAYER_LOGIN()
    hcTk.DebugPrint("HC Login")
    hcTk.GroupFill()
    hcTk.CommCheck()
end

function HCToolkit:PLAYER_SPECIALIZATION_CHANGED()
    hcTk.SpecializationSend(UnitGUID("player"))
end

function HCToolkit:ChatCommand(input)
    if (string.find(input, "versionGroup")) then
        hcTk.VersionPrint()
    elseif (string.find(input, "version")) then
        hcTk:Print("Version: " .. hcTk.Constants.Version)
    elseif (string.find(input, "assignments")) then
        hcTk.AssignmentPanel()
    else
        InterfaceOptionsFrame_OpenToCategory(hcTk.OptionsFrame)
        InterfaceOptionsFrame_OpenToCategory(hcTk.OptionsFrame)
    end
end

function HCToolkit:CoreComm(prefix, input, distribution, sender)
    local status, data = hcTk:Deserialize(input)
    if (data["type"] == "VersionSend" and data["guid"] ~= nil and data["version"] ~= nil) then
        hcTk.DebugPrint("Received Version For " .. data["guid"] .. " | " .. data["version"])
        hcTk.UnitUpdateVersion(data["guid"], data["version"])
    elseif (data["type"] == "VersionRequest") then
        hcTk.VersionSend(UnitGUID("player"))
    elseif (data["type"] == "CommCheck" and not hcTk.CommsWorking) then
        hcTk.DebugPrint("Comms Working")
        hcTk.CommsWorking = true
        C_Timer.After(1, hcTk.RequestInformation)
    end
end

function HCToolkit:BossComm(prefix, input, distribution, sender)
    if (hcTk.EncounterId == nil) then return nil end
    
    local status, data = hcTk:Deserialize(input)
    if (data["type"] == "AvatarTaintedSoakGroups" and data["TaintedSoakLeft"] ~= nil and data["TaintedSoakRight"] ~= nil) then
        hcTk.AvatarReceiveTaintedSoakGroups(data["TaintedSoakLeft"], data["TaintedSoakRight"])
    elseif (data["type"] == "KilJaedenSendSmallArmageddonGroups") then
        hcTk.KilJaedenReceiveSmallArmageddonGroups(data)
    elseif (data["type"] == "KilJaedenSendBigArmageddonGroups") then
        hcTk.KilJaedenReceiveBigArmageddonGroups(data)
    end
end

function HCToolkit:FlameComm(prefix, input, distribution, sender)
    if ((distribution == "PARTY" or distribution == "RAID") and hcTk.GenericGet("MattySmite")) then
        PlaySoundFile("Interface\\AddOns\\HCToolkit\\Media\\ThatsWrong.mp3", "Master")
    end
end

function HCToolkit:RosterComm(prefix, input, distribution, sender)
    local status, data = hcTk:Deserialize(input)
    if (data["type"] == "SpecializationSend" and data["guid"] ~= nil and data["specId"] ~= nil) then
        hcTk.UnitUpdateRole(data["guid"], data["specId"])
    elseif (data["type"] == "SpecializationRequest") then
        hcTk.SpecializationSend(UnitGUID("player"))
    end
end