local hcTk = HCToolkit

hcTk.FramesInit = function()
    HCToolkitGroupFrame = CreateFrame("Frame", "HCToolkitGroupFrame", UIParent);
    HCToolkitGroupFrame.MinHeight = 20
    HCToolkitGroupFrame.MinWidth = 200
    HCToolkitGroupFrame.Unlocked = false
    HCToolkitPersonalFrame = CreateFrame("Frame", "HCToolkitPersonalFrame", UIParent);
    HCToolkitPersonalFrame.MinHeight = 20
    HCToolkitPersonalFrame.MinWidth = 400
    HCToolkitPersonalFrame.Unlocked = false

    hcTk.CreateFrame(HCToolkitGroupFrame, hcTk.DB.profile.Frames.Group)
    hcTk.CreateFrame(HCToolkitPersonalFrame, hcTk.DB.profile.Frames.Personal)
end

hcTk.CalculatePoint = function(index, ogPoint, beforePoint, afterPoint)
    return ogPoint[index] + (afterPoint[index] - beforePoint[index])
end

hcTk.CreateFrame = function(frame, settings)
    frame.DemoMode = false
    frame.texture = frame:CreateTexture(nil, "BACKGROUND")
    frame.font = frame:CreateFontString(nil, "OVERLAY")
    frame.settings = settings

    frame:SetScript("OnMouseDown", function(self, button)
        if frame.Unlocked and button == "LeftButton" and not self.isMoving then
            self.ogPoint = { frame:GetPoint(1) }
            self:StartMoving();
            self.beforePoint = { frame:GetPoint(1) }
            self.isMoving = true;
        end
    end)

    frame:SetScript("OnMouseUp", function(self, button)
        if button == "LeftButton" and self.isMoving then
            local xOffset = hcTk.CalculatePoint(4, self.ogPoint, self.beforePoint, { frame:GetPoint(1) })
            local yOffset = hcTk.CalculatePoint(5, self.ogPoint, self.beforePoint, { frame:GetPoint(1) })
            self:StopMovingOrSizing()
            self.isMoving = false

            self:ClearAllPoints()
            self:SetPoint(settings.Anchor, nil, "CENTER", xOffset, yOffset)

            self.ogPoint = nil
            self.beforePoint = nil

            settings.X = xOffset
            settings.Y = yOffset
        end
    end)

    frame:SetScript("OnHide", function(self)
        if ( self.isMoving ) then
            self:StopMovingOrSizing();
            self.isMoving = false;
        end
    end)

    frame:SetHeight(frame.MinHeight)
    frame:SetWidth(frame.MinWidth)
    frame:SetPoint(settings.Anchor, nil, "CENTER", settings.X, settings.Y)
    frame:SetMovable(frame.Unlocked)
    frame:EnableMouse(frame.Unlocked)

    frame.font:SetAllPoints(true)
    frame.font:SetJustifyH(settings.Text.Align)
    frame.font:SetJustifyV("TOP")
    frame.font:SetFont(hcTk.LSM:Fetch("font", settings.Text.Font), settings.Text.Size, settings.Text.Flag)
    frame.font:SetTextColor(1, 1, 1, 1)

    frame.texture:SetAllPoints(frame)
    frame.texture:SetTexture(0, 0, 0, 0)

    -- We double resize on the next frame update, styled text has issues that requires a render
    frame:SetScript("OnSizeChanged", function(self) C_Timer.After(0, function() hcTk.ResizeFrame(self); end) end)
end

hcTk.ResizeFrame = function(frame)
    local height = frame.font:GetStringHeight()
    local width = frame.font:GetStringWidth()

    if (height == nil or frame.MinHeight >= height) then
        height = frame.MinHeight
    end

    if (width == nil or frame.MinWidth >= width) then
        width = frame.MinWidth
    end

    frame:SetHeight(height)
    frame:SetWidth(width)
end

hcTk.FrameUpdateText = function(frame, text)
    frame.font:SetText(text)
    hcTk.ResizeFrame(frame)
end

hcTk.FrameDemoMode = function(frame, v)
    local text = ""

    if (v) then
        if (frame == HCToolkitGroupFrame) then
            text = text .. string.format("%s", "Outer | Middle | Inner\n")
            text = text .. string.format("|c%s%s:|r %s | %s | %s\n", "FF00FF00", "Fel Front", "Clutch", "Chaap", "Nastynelbur")
            text = text .. string.format("|c%s%s:|r %s | %s | %s\n", "FF00FF00", "Fel Back", "Auro", "Jetshunter", "Tenkiei")
            text = text .. string.format("|c%s%s:|r %s | %s | %s\n", "FFFFDF00", "Holy Front", "Ugly", "Nightsdh", "Buddie")
            text = text .. string.format("|c%s%s:|r %s | %s | %s", "FFFFDF00", "Holy Back", "Partyrawk", "Skyline", "Procz")
        elseif (frame == HCToolkitPersonalFrame) then
            -- text = string.format("|c%s%s|r", "FFFFDF00", "INNER - FRONT")
            text = string.format("%s", "|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_3:0|t Test")
        end
    end

    frame.DemoMode = v
    hcTk.FrameUpdateText(frame, text)
end

hcTk.FrameResetState = function(frame)
    frame:SetScript("OnUpdate", nil)
    hcTk.FrameUpdateText(frame, "")
end